package net.knarcraft.permissionsigns.listener;

import net.knarcraft.permissionsigns.PermissionSigns;
import net.knarcraft.permissionsigns.container.SignCreationRequest;
import net.knarcraft.permissionsigns.formatting.TranslatableMessage;
import net.knarcraft.permissionsigns.formatting.Translator;
import org.bukkit.Material;
import org.bukkit.Tag;
import org.bukkit.block.Block;
import org.bukkit.block.Sign;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.player.PlayerInteractEvent;

import java.util.Arrays;

/**
 * A listener for all events related to signs
 */
public class SignListener implements Listener {

    /**
     * This event handler detects if a player clicks a button or a sign
     *
     * @param event <p>The player interact event which was triggered</p>
     */
    @EventHandler
    public void onPlayerInteract(PlayerInteractEvent event) {
        Player player = event.getPlayer();
        Block block = event.getClickedBlock();

        if (block == null) {
            return;
        }

        Material material = block.getBlockData().getMaterial();
        if (!Tag.SIGNS.isTagged(material) && !Tag.WALL_SIGNS.isTagged(material)) {
            return;
        }

        Sign sign = (Sign) block.getState();
        if (event.getAction() == Action.RIGHT_CLICK_BLOCK) {
            handleSignRightClick(sign, player);
        } else if (event.getAction() == Action.LEFT_CLICK_BLOCK) {
            handleSignLeftClick(sign, player, event);
        }
    }

    private void handleSignLeftClick(Sign sign, Player player, PlayerInteractEvent event) {
        //TODO: Check if the sign is a registered permissions sign
        boolean registered = true;
        if (!registered) {
            return;
        }
        if (!player.hasPermission("permissionsigns.admin")) {
            event.setCancelled(true);
            player.sendMessage(Translator.getTranslatedMessage(TranslatableMessage.PERMISSION_SIGN_DESTROY_DENY));
        } else {
            //TODO: Un-register the permissions sign
            player.sendMessage(Translator.getTranslatedMessage(TranslatableMessage.PERMISSION_SIGN_REMOVED));
        }
    }

    /**
     * Handles the right-clicking action
     *
     * @param sign   <p>The clicked sign</p>
     * @param player <p>The player that clicked the sign</p>
     */
    private void handleSignRightClick(Sign sign, Player player) {
        String[] lines = sign.getLines();

        //TODO: Check if the sign is a registered permissions sign

        //Don't allow non-empty signs to be overwritten
        if (!Arrays.stream(lines).allMatch(String::isEmpty)) {
            return;
        }

        SignCreationRequest request = PermissionSigns.getSignCreationRequest(player.getUniqueId());
        if (request == null) {
            return;
        }

        //TODO: Register the sign and remove the request
    }

}
